"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class CommandInfo {
    constructor(command) {
        this.aliases = [];
        this.shortDescription = "";
        this.longDescription = "";
        this.syntax = "";
        this.showInHelp = true;
        this.caseInsensitive = true;
        this.deleteCommand = false;
        this.context = "Any";
        this.parameters = [];
        this.requirements = {};
        const metadata = Reflect.getMetadata("command", command);
        for (const prop in metadata) {
            if (metadata.hasOwnProperty(prop)) {
                this[prop] = metadata[prop];
            }
        }
        this.buildSyntax();
    }
    get name() {
        return this.aliases[0];
    }
    buildSyntax() {
        let message = "";
        message += `{prefix}${this.name} `;
        for (const parameter of this.parameters) {
            message += parameter.required ? "<" : "[";
            if (parameter.multiple || parameter.remainder) {
                message += "...";
            }
            message += parameter.name;
            if (parameter.defaultValue !== undefined && parameter.defaultValue !== null) {
                message += "=" + JSON.stringify(parameter.defaultValue);
            }
            message += parameter.required ? "> " : "] ";
        }
        this.syntax = message.toString().trim();
    }
}
exports.default = CommandInfo;
//# sourceMappingURL=CommandInfo.js.map