"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const eventemitter3_1 = require("eventemitter3");
const fs = require("fs");
const inversify_1 = require("inversify");
const path = require("path");
const typeorm_1 = require("typeorm");
const winston_1 = require("winston");
const CommandHandler_1 = require("./Command/CommandHandler");
const CommandHelper_1 = require("./Command/CommandHelper");
const CommandInfoFactory_1 = require("./Factory/CommandInfoFactory");
const GatewayFactory_1 = require("./Factory/GatewayFactory");
const GatewayService_1 = require("./GatewayService");
const types_1 = require("./types");
class ContainerBuilder {
    static build(kernel, config) {
        return __awaiter(this, void 0, void 0, function* () {
            const container = new inversify_1.Container({ defaultScope: "Singleton" });
            container.bind(types_1.default.Kernel).toConstantValue(kernel);
            container.bind(types_1.default.Logger).toConstantValue(ContainerBuilder.CreateLogger.call(this));
            container.bind(types_1.default.Configuration).toConstantValue(config);
            container.bind(types_1.default.EventEmitter).toConstantValue(new eventemitter3_1.EventEmitter());
            if (config.database) {
                const connection = yield typeorm_1.createConnection(Object.assign({ type: "mysql", entities: [__dirname + "/Entity/*"] }, config.database.mysql));
                container.bind(types_1.default.Connection).toConstantValue(connection);
            }
            container.bind("Container").toConstantValue(container);
            container.bind(types_1.default.Command.Handler).to(CommandHandler_1.default);
            container.bind(types_1.default.Factory.Info).to(CommandInfoFactory_1.default);
            container.bind(types_1.default.GatewayService).to(GatewayService_1.default);
            container.bind(types_1.default.Factory.Gateway).to(GatewayFactory_1.default);
            container.bind(types_1.default.Command.Helper).to(CommandHelper_1.default);
            let files;
            try {
                files = fs.readdirSync("./Command");
            }
            catch (e) {
                console.log("No commands!");
            }
            let commandClient = false;
            if (files) {
                for (const file of files) {
                    if (path.extname(file) !== ".js" || file.indexOf("Abstract") >= 0) {
                        continue;
                    }
                    commandClient = true;
                    const cls = require(`./Command/${file}`).default;
                    const info = container.get(types_1.default.Factory.Info).getCommandInfo(cls);
                    container.bind(types_1.default.Command.Interface)
                        .to(cls)
                        .inTransientScope()
                        .whenTargetNamed(info.name);
                }
            }
            config.gateways.forEach((url) => __awaiter(this, void 0, void 0, function* () {
                container.bind(types_1.default.Gateway)
                    .toDynamicValue((context) => {
                    const factory = context.container.get(types_1.default.Factory.Gateway);
                    return factory.create(url);
                });
            }));
            return container;
        });
    }
    static CreateLogger() {
        const logger = new winston_1.Logger({
            exitOnError: false,
            level: process.env.NODE_ENV === "production" ? "debug" : "silly",
            transports: [
                new winston_1.transports.Console({
                    colorize: true,
                    handleExceptions: true,
                    humanReadableUnhandledException: true,
                    prettyPrint: true,
                    silent: false,
                    timestamp: true,
                }),
            ],
        });
        logger.cli();
        return logger;
    }
}
exports.ContainerBuilder = ContainerBuilder;
//# sourceMappingURL=ContainerBuilder.js.map